from persian_tools import separator


def test_add_separator():
    assert separator.add(1000000) == '1,000,000'
    assert separator.add(100000.0) == '100,000.0'
    assert separator.add(100000.0001) == '100,000.0001'
    assert separator.add('some text 1000000') == 'some text 1,000,000'
    assert separator.add('۱۲۳۴۵۶۷۸۹۰', '٫') == '۱٫۲۳۴٫۵۶۷٫۸۹۰'
    assert separator.add('١٢٣٤٥٦٧٨٩٠') == '١,٢٣٤,٥٦٧,٨٩٠'

    assert separator.add('') == ''
    assert separator.add(0) == '0'


def test_remove_separator():
    assert separator.remove('1,000,000') == '1000000'
    assert separator.remove('100,000.0') == '100000.0'
    assert separator.remove('some text 1,000,000') == 'some text 1000000'
    assert separator.remove('۱٫۲۳۴٫۵۶۷٫۸۹۰', '٫') == '۱۲۳۴۵۶۷۸۹۰'
    assert separator.remove('١,٢٣٤,٥٦٧,٨٩٠') == '١٢٣٤٥٦٧٨٩٠'



from persian_tools.bank import sheba
from persian_tools.bank.exceptions import InvalidShebaNumber
import pytest


def test_validate():
    assert sheba.validate('IR820540102680020817909002')
    assert sheba.validate('IR550570022080013447370101')

    assert sheba.validate('IR01234567890123456789') is False
    assert sheba.validate('IR012345678901234567890123456789') is False
    assert sheba.validate('IR01234567890123456789') is False
    assert sheba.validate('IR012345678901234567890123') is False
    assert sheba.validate('IR012345678901234567890123') is False
    assert sheba.validate('01234567890123456789012345') is False


def test_bank_data():
    actual = sheba.bank_data('IR820540102680020817909002')
    expected = {
        'nickname': 'parsian',
        'name': 'Parsian Bank',
        'persian_name': 'بانک پارسیان',
        'card_prefix': ['622106', '627884'],
        'sheba_code': ['054'],
        'account_number': '020817909002',
        'formatted_account_number': '002-00817909-002'
    }
    assert all(v == actual[k] for k, v in expected.items())
    assert len(expected) == len(actual)

    actual = sheba.bank_data('IR550570022080013447370101')
    expected = {
        'nickname': 'pasargad',
        'name': 'Pasargad Bank',
        'persian_name': 'بانک پاسارگاد',
        'card_prefix': ['502229', '639347'],
        'sheba_code': ['057'],
        'account_number': '220800134473701',
        'formatted_account_number': '220-800-13447370-1'
    }
    assert all(v == actual[k] for k, v in expected.items())
    assert len(expected) == len(actual)

    actual = sheba.bank_data('IR790610000000700796858044')
    expected = {
        'nickname': 'shahr',
        'name': 'City Bank',
        'persian_name': 'بانک شهر',
        'card_prefix': ['502806', '504706'],
        'sheba_code': ['061'],
        'account_number': '700796858044',
        'formatted_account_number': '700796858044'
    }
    assert all(v == actual[k] for k, v in expected.items())
    assert len(expected) == len(actual)

    with pytest.raises(InvalidShebaNumber, match='.*is an invalid sheba number'):
        sheba.bank_data('IR012345678901234567890123')
        sheba.bank_data('IR012345678A01234567890123')



from persian_tools import digits


def test_convert_to_fa():
    assert digits.convert_to_fa('123٤٥٦') == '۱۲۳۴۵۶'
    assert digits.convert_to_fa(123456) == '۱۲۳۴۵۶'


def test_convert_to_ar():
    assert digits.convert_to_ar('123۴۵۶') == '١٢٣٤٥٦'
    assert digits.convert_to_ar(123.456) == '١٢٣.٤٥٦'


def test_convert_to_en():
    assert digits.convert_to_en('۱۲۳٤٥٦') == '123456'


def test_chaining_behavior():
    assert digits.convert_to_en(digits.convert_to_fa('123٤٥٦')) == '123456'


def test_conversion_function():
    assert digits._conversion('123', 'de') is None


def test_convert_to_word():
    assert digits.convert_to_word(500443) == 'پانصد هزار و چهارصد و چهل و سه'
    assert digits.convert_to_word(987654321) == \
           'نهصد و هشتاد و هفت میلیون و ششصد و پنجاه و چهار هزار و سیصد و بیست و یک'
    assert len(digits.convert_to_word(500)) == 5
    assert digits.convert_to_word(30000000000) == 'سی میلیارد'
    assert digits.convert_to_word(30000000000000) == 'سی بیلیون'
    assert digits.convert_to_word(30000000000000000) == 'سی بیلیارد'
    assert digits.convert_to_word(300000000000000000000) is None
    assert digits.convert_to_word(0) == 'صفر'
    assert digits.convert_to_word(4) == 'چهار'
    assert digits.convert_to_word(33) == 'سی و سه'

    assert digits.convert_to_word(500443, ordinal=True) == 'پانصد هزار و چهارصد و چهل و سوم'
    assert digits.convert_to_word(-30, ordinal=True) == 'منفی سی اُم'
    assert digits.convert_to_word(-123, ordinal=True) == 'منفی صد و بیست و سوم'
    assert digits.convert_to_word(33, ordinal=True) == 'سی و سوم'
    assert digits.convert_to_word(45, ordinal=True) == 'چهل و پنجم'


def test_convert_from_word():
    assert digits.convert_from_word('') is None
    assert digits.convert_from_word(None) is None
    assert digits.convert_from_word('متن بدون عدد') == 0
    assert digits.convert_from_word('صفر') == 0

    assert digits.convert_from_word('منفی سه هزار') == -3000
    assert digits.convert_from_word('سه هزار دویست و دوازده') == 3212
    assert digits.convert_from_word('دوازده هزار بیست دو') == 12022
    assert digits.convert_from_word('دوازده هزار بیست دو', separator=True) == '12,022'
    assert digits.convert_from_word('دوازده هزار و بیست و دو', separator=True) == '12,022'
    assert digits.convert_from_word('شیش صد و بیست و هفت') == 627
    assert digits.convert_from_word('حقوق شیش صد و ۲۷ میلیون تومان سالانه') == 627 * 1000 * 1000


def test_convert_from_word_to_ar():
    assert digits.convert_from_word("منفی سه هزار", digits="ar") == "-٣٠٠٠"
    assert digits.convert_from_word("سه هزار دویست و دوازده", digits="ar") == "٣٢١٢"
    assert digits.convert_from_word("دوازده هزار بیست دو", digits="ar") == "١٢٠٢٢"
    assert digits.convert_from_word("دوازده هزار بیست دو", digits="ar", separator=True) == "١٢,٠٢٢"
    assert digits.convert_from_word("دوازده هزار و بیست و دو", digits="ar", separator=True) == "١٢,٠٢٢"
    assert digits.convert_from_word("چهارصد پنجاه هزار", digits="ar", separator=True) == "٤٥٠,٠٠٠"
    assert digits.convert_from_word("چهارصد پنجاه هزار", digits="ar") == "٤٥٠٠٠٠"


def test_convert_from_word_with_ordinal():
    assert digits.convert_from_word("منفی ۳ هزار", digits="fa", separator=True) == "-۳,۰۰۰"
    assert digits.convert_from_word("منفی 3 هزار و 200", digits="fa", separator=True) == "-۳,۲۰۰"
    assert digits.convert_from_word("منفی سه هزارمین", digits="fa", separator=True) == "-۳,۰۰۰"
    assert digits.convert_from_word("منفی سه هزارمین", digits="fa") == "-۳۰۰۰"
    assert digits.convert_from_word("منفی سه هزارمین") == -3000
    assert digits.convert_from_word("منفی سه هزارم") == -3000
    assert digits.convert_from_word("منفی سه هزارمین") != "-3000"
    assert len(str(digits.convert_from_word("منفی سه هزارمین"))) == 5
    assert digits.convert_from_word("منفی سی اُم") == -30
    assert digits.convert_from_word("سی و سوم") == 33



import re
from typing import Union
from .operators import data as operators_data
from .exceptions import InvalidPhoneNumber, InvalidToken

MOBILE_REGEX = '(?:[+|0{2}]?98)?(?:0)?(\\d{3})+(\\d{3})+(\\d{4})'
VALID_TOKENS = ['+98', '98', '0098', '0']
MOBILE_SUFFIX_REGEX = '^(?:\+98|98|0098|0)'


def _get_operator_data(phone_number: str) -> Union[dict, bool]:
    phone_number_suffix = re.split(MOBILE_SUFFIX_REGEX, phone_number).pop()
    for prefix_length in range(3, 6):
        operator_prefix = phone_number_suffix[:prefix_length]
        if operators_data.get(operator_prefix):
            return operators_data.get(operator_prefix)

    return False


def validate(phone_number: str) -> bool:
    return _get_operator_data(phone_number) is not False


def operator_data(phone_number: str) -> bool:
    res = _get_operator_data(phone_number)
    if res is False:
        raise InvalidPhoneNumber(phone_number)

    return res


def normalize(phone_number: str, token: str = '0') -> str:
    if token not in VALID_TOKENS:
        raise InvalidToken(token)

    valid = validate(phone_number)
    if valid is False:
        raise InvalidPhoneNumber(phone_number)

    phone_number_suffix = re.split(MOBILE_SUFFIX_REGEX, phone_number).pop()
    return token + phone_number_suffix



class InvalidPhoneNumber(Exception):
    def __init__(self, phone_number):
        self.phone_number = phone_number
        self.message = phone_number + ' is an invalid phone number'
        super().__init__(self.message)


class InvalidToken(Exception):
    def __init__(self, token):
        self.token = token
        self.message = token + ' is an invalid token'
        super().__init__(self.message)



import random

_COEFFICIENT = [29, 27, 23, 19, 17]


def validate(legal_id: str) -> bool:
    code = legal_id.zfill(11)

    if not code.isdigit() or len(code) != 11:
        return False

    _constant = int(code[-2]) + 2
    _sum = 0
    for i in range(10):
        _sum += (int(code[i]) + int(_constant)) * _COEFFICIENT[i % 5]

    remainder = 0 if _sum % 11 == 10 else _sum % 11

    return remainder == int(code[-1])


def generate_random() -> str:
    random_number = random.randint(10 ** 8, 10 ** 10 - 1)
    random_number = str(random_number).zfill(10)

    _constant = int(random_number[-1]) + 2
    _sum = 0
    for i in range(10):
        _sum += (int(random_number[i]) + int(_constant)) * _COEFFICIENT[i % 5]

    remainder = _sum % 11
    if _sum % 11 == 10:
        remainder = 0

    return random_number + str(remainder)



from persian_tools import legal_id


def test_validate_legal_id():
    # These legal ids are valid and registered
    assert legal_id.validate('10380284790')
    assert legal_id.validate('10100387143')

    # These legal ids are all valid
    assert legal_id.validate('77111986110')
    assert legal_id.validate('18061055669')
    assert legal_id.validate('15292247120')
    assert legal_id.validate('42118547230')
    assert legal_id.validate('27388385445')

    assert legal_id.validate('55828465992')
    assert legal_id.validate('32882926778')
    assert legal_id.validate('97006806007')
    assert legal_id.validate('21272129339')
    assert legal_id.validate('18706641333')

    # These legal ids are all invalid
    assert legal_id.validate('84735069516') is False
    assert legal_id.validate('65399567978') is False
    assert legal_id.validate('99969101031') is False
    assert legal_id.validate('15826563190') is False
    assert legal_id.validate('11053639140') is False
    assert legal_id.validate('18090089802') is False
    assert legal_id.validate('123001000') is False
    assert legal_id.validate('11111111111') is False
    assert legal_id.validate('10380284792') is False
    assert legal_id.validate('10380285692') is False
    assert legal_id.validate('09748208301') is False
    assert legal_id.validate('097482083013') is False
    assert legal_id.validate('0974820830a') is False


def test_generate_random_legal_id():
    for _ in range(15):
        assert legal_id.validate(legal_id.generate_random())



from persian_tools import phone_number
from persian_tools.phone_number.exceptions import InvalidPhoneNumber, InvalidToken
import pytest


def test_validate():
    assert phone_number.validate('09022002580')
    assert phone_number.validate('09122002580')
    assert phone_number.validate('09322002580')
    assert phone_number.validate('09192002580')
    assert phone_number.validate('+989022002580')
    assert phone_number.validate('09022002580')
    assert phone_number.validate('989022002580')
    assert phone_number.validate('00989022002580')
    assert phone_number.validate('9022002580')

    assert phone_number.validate('09802002580') is False
    assert phone_number.validate('0980200') is False


def test_operator_data():
    actual = phone_number.operator_data('09022002580')
    expected = {'province': [], 'base': 'کشوری', 'type': ['permanent', 'credit'],'operator': 'ایرانسل'}
    assert all(v == actual[k] for k, v in expected.items())
    assert len(expected) == len(actual)

    actual = phone_number.operator_data('09981000000')
    expected = {'province': [], 'base': 'کشوری', 'type': ['credit'], 'operator': 'شاتل موبایل'}
    assert all(v == actual[k] for k, v in expected.items())
    assert len(expected) == len(actual)

    actual = phone_number.operator_data('09300880440')
    expected = {'province': [], 'base': 'کشوری', 'type': ['permanent', 'credit'], 'operator': 'ایرانسل'}
    assert all(v == actual[k] for k, v in expected.items())
    assert len(expected) == len(actual)

    actual = phone_number.operator_data('09999980440')
    expected = {'province': [], 'base': 'کشوری', 'type': ['credit', 'permanent'], 'operator': 'سامانتل'}
    assert all(v == actual[k] for k, v in expected.items())
    assert len(expected) == len(actual)

    with pytest.raises(InvalidPhoneNumber, match='.*is an invalid phone number'):
        phone_number.operator_data('+989990467927')


def test_normalize():
    assert phone_number.normalize('+989373708555', '0') == '09373708555'
    assert phone_number.normalize('989373708555', '0') == '09373708555'
    assert phone_number.normalize('00989022002580', '0') == '09022002580'
    assert phone_number.normalize('09122002580', '0') == '09122002580'
    assert phone_number.normalize('9322002580', '0') == '09322002580'

    assert phone_number.normalize('09373708555', '+98') == '+989373708555'
    assert phone_number.normalize('09022002580', '+98') == '+989022002580'
    assert phone_number.normalize('09122002580', '+98') == '+989122002580'
    assert phone_number.normalize('9322002580', '+98') == '+989322002580'
    assert phone_number.normalize('00989022002580', '+98') == '+989022002580'

    assert phone_number.normalize('9322002580', '+98') == '+989322002580'
    assert phone_number.normalize('9322002580', '98') == '989322002580'
    assert phone_number.normalize('9322002580', '0098') == '00989322002580'
    assert phone_number.normalize('9322002580', '0') == '09322002580'
    assert phone_number.normalize('9322002580') == '09322002580'

    with pytest.raises(InvalidPhoneNumber, match='.*is an invalid phone number'):
        phone_number.normalize('0', '+98')
        phone_number.normalize('+98', '+98')
        phone_number.normalize('99999999999', '+98')
        phone_number.normalize('   99999999     ', '+98')
        phone_number.normalize('09802002580', '0')

    with pytest.raises(InvalidToken, match='.*is an invalid token'):
        phone_number.normalize('9322002580', '098')
